####################
# filename: AxesTagging2D.tcl
# author: Hanlin DONG
# description: Utility functions to use AxesTagging strategy
# create: 2019-05-07 15:41:33
# license: MIT License (https://opensource.org/licenses/MIT)
# version: 1.0
####################

# console log
puts "AxesTagging2D.tcl is successfully imported. version 1.0. (Author: Hanlin Dong. MIT License.)"

# from a axes distance list, assemble a coordinate list.
proc AssembleCoordsList {dList} {
    set coords [list 0.0]
    for {set i 0} {$i < [llength $dList]} {incr i} {
        lappend coords [expr [lindex $coords $i] + [lindex $dList $i]]
    }
    return $coords
}

# Create node on each intersection of the axes list. TAG rules:
#  e.g.  1           01                21         = 10121
#    (prefix) - (i Axis Number) - (j Axis Number)
proc CreateNodes {prefix iList jList XCoords YCoords} {
    foreach i $iList {
        foreach j $jList {
            node [expr $prefix * 10000 + $i * 100 + $j] [lindex $XCoords $i] [lindex $YCoords $j]
        }
    }
}

# Fix the nodes using the fixList whose length is 3 and components are 0 or 1
#  e.g. fixList = { 1 1 1 }
proc FixNodes {prefix iList jList fixList} {
    foreach i $iList {
        foreach j $jList {
            fix [expr $prefix*10000 + $i*100 + $j] [lindex $fixList 0] [lindex $fixList 1] [lindex $fixList 2]
        }
    }
}

# Getter
proc GetNodeTag { prefix i j } {
    return [expr $prefix * 10000 + $i * 100 + $j]
}

# Create vertical elements. iList is discrete. jList should be continuous.
# TAG rule:
#  e.g.    1             11                 01                 02         = 1110102
#       (prefix) - (i Axis Number) - (j Axis Number 1) - (j Axis Number 2)
proc CreateVerticalEles { prefix iList jList } {
    set result {}
    foreach i $iList {
        for { set n 1 } { $n < [llength $jList] } { incr n } {
            set j [lindex $jList $n]
            set jPrev [lindex $jList [expr $n - 1]]
            set eleTag [expr $prefix * 1000000 + $i * 10000 + $jPrev * 100 + $j]
            set nodeTag1 [expr $prefix * 10000 + $i * 100 + $jPrev]
            set nodeTag2 [expr $prefix * 10000 + $i * 100 + $j]
            lappend result [list $eleTag $nodeTag1 $nodeTag2]
        }
    }
    return $result
}

# Create horizontal elements. jList is discrete. iList should be continuous.
# TAG rule:
#  e.g.    2              11                 12                 01         = 2111201
#       (prefix) - (i Axis Number 1) - (i Axis Number 2) - (j Axis Number)
proc CreateHorizontalEles { prefix iList jList } {
    set result {}
    foreach j $jList {
        for { set m 1 } { $m < [llength $iList] } { incr m } {
            set i [lindex $iList $m]
            set iPrev [lindex $iList [expr $m - 1]]
            set eleTag [expr $prefix * 1000000 + $iPrev * 10000 + $i * 100 + $j]
            set nodeTag1 [expr $prefix * 10000 + $iPrev * 100 + $j]
            set nodeTag2 [expr $prefix * 10000 + $i * 100 + $j]
            lappend result [list $eleTag $nodeTag1 $nodeTag2]
        }
    }
    return $result
}

# Create zerolength elements on every intersection of i and j axes listed.
# TAG rule:
#  e.g.      1            1           2             02               12         = 1120212
#       (prefixEle) - (prefix1) - (prefix2) - (i Axis Number) - (j Axis Number)
proc CreateZerolengthEles { prefixEle prefix1 prefix2 iList jList } {
    set result {}
    foreach i $iList {
        foreach j $jList {
            set eleTag [expr $prefixEle * 1000000 + $prefix1 * 100000 + $prefix2 * 10000 + $i * 100 + $j]
            set nodeTag1 [expr $prefix1 * 10000 + $i * 100 + $j]
            set nodeTag2 [expr $prefix2 * 10000 + $i * 100 + $j]
            lappend result [list $eleTag $nodeTag1 $nodeTag2]
        }
    }
    return $result
}